package sun.security.provider.certpath;

import java.util.Map;
import java.security.cert.CRLReason;
import sun.security.x509.GeneralName;
import sun.security.x509.AuthorityInfoAccessExtension;
import sun.security.x509.URIName;
import sun.security.x509.AccessDescription;
import java.net.URL;
import java.io.OutputStream;
import java.io.InputStream;
import java.util.Arrays;
import java.net.HttpURLConnection;
import java.util.Iterator;
import sun.security.x509.PKIXExtensions;
import java.security.cert.CertificateException;
import sun.security.x509.X509CertImpl;
import java.security.cert.TrustAnchor;
import java.security.cert.Extension;
import java.util.List;
import java.security.cert.CertPathValidatorException;
import java.io.IOException;
import java.util.Collections;
import java.util.Date;
import java.net.URI;
import java.security.cert.X509Certificate;
import java.security.AccessController;
import sun.security.action.GetIntegerAction;
import sun.security.util.Debug;

public final class OCSP
{
    private static final Debug debug;
    private static final int DEFAULT_CONNECT_TIMEOUT = 15000;
    private static final int CONNECT_TIMEOUT;
    
    private static int initializeTimeout() {
        Integer n = (Integer)AccessController.doPrivileged(new GetIntegerAction("com.sun.security.ocsp.timeout"));
        if (n == null || (int)n < 0) {
            return 15000;
        }
        return (int)n * 1000;
    }
    
    private OCSP() {
    }
    
    public static OCSP.RevocationStatus check(X509Certificate x509Certificate, X509Certificate x509Certificate2, URI uri, X509Certificate x509Certificate3, Date date) throws IOException, CertPathValidatorException {
        return check(x509Certificate, x509Certificate2, uri, x509Certificate3, date, Collections.emptyList(), "generic");
    }
    
    public static OCSP.RevocationStatus check(X509Certificate x509Certificate, X509Certificate x509Certificate2, URI uri, X509Certificate x509Certificate3, Date date, List<Extension> list, String s) throws IOException, CertPathValidatorException {
        return check(x509Certificate, uri, null, x509Certificate2, x509Certificate3, date, list, s);
    }
    
    public static OCSP.RevocationStatus check(X509Certificate cert, URI uri, TrustAnchor trustAnchor, X509Certificate x509Certificate, X509Certificate x509Certificate2, Date date, List<Extension> list, String s) throws IOException, CertPathValidatorException {
        CertId o;
        try {
            o = new CertId(x509Certificate, X509CertImpl.toImpl(cert).getSerialNumberObject());
        }
        catch (CertificateException | IOException cause) {
            throw new CertPathValidatorException("Exception while encoding OCSPRequest", cause);
        }
        return check(Collections.singletonList(o), uri, new OCSPResponse.IssuerInfo(trustAnchor, x509Certificate), x509Certificate2, date, list, s).getSingleResponse(o);
    }
    
    static OCSPResponse check(List<CertId> list, URI uri, OCSPResponse.IssuerInfo issuerInfo, X509Certificate x509Certificate, Date date, List<Extension> list2, String s) throws IOException, CertPathValidatorException {
        byte[] value = null;
        for (Extension extension : list2) {
            if (extension.getId().equals(PKIXExtensions.OCSPNonce_Id.toString())) {
                value = extension.getValue();
            }
            else {
                continue;
            }
        }
        Object o;
        try {
            o = new OCSPResponse(getOCSPBytes(list, uri, list2));
            o.verify(list, issuerInfo, x509Certificate, date, value, s);
        }
        catch (IOException cause) {
            throw new CertPathValidatorException("Unable to determine revocation status due to network error", cause, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
        }
        return o;
    }
    
    public static byte[] getOCSPBytes(List<CertId> list, URI uri, List<Extension> list2) throws IOException {
        byte[] encodeBytes = new OCSPRequest(list, list2).encodeBytes();
        InputStream inputStream = null;
        OutputStream outputStream = null;
        byte[] original = null;
        try {
            URL url = uri.toURL();
            if (OCSP.debug != null) {
                OCSP.debug.println("connecting to OCSP service at: " + url);
            }
            HttpURLConnection httpURLConnection = (HttpURLConnection)url.openConnection();
            httpURLConnection.setConnectTimeout(OCSP.CONNECT_TIMEOUT);
            httpURLConnection.setReadTimeout(OCSP.CONNECT_TIMEOUT);
            httpURLConnection.setDoOutput(true);
            httpURLConnection.setDoInput(true);
            httpURLConnection.setRequestMethod("POST");
            httpURLConnection.setRequestProperty("Content-type", "application/ocsp-request");
            httpURLConnection.setRequestProperty("Content-length", String.valueOf(encodeBytes.length));
            outputStream = httpURLConnection.getOutputStream();
            outputStream.write(encodeBytes);
            outputStream.flush();
            if (OCSP.debug != null && httpURLConnection.getResponseCode() != 200) {
                OCSP.debug.println("Received HTTP error: " + httpURLConnection.getResponseCode() + " - " + httpURLConnection.getResponseMessage());
            }
            inputStream = httpURLConnection.getInputStream();
            int contentLength = httpURLConnection.getContentLength();
            if (contentLength == -1) {
                contentLength = Integer.MAX_VALUE;
            }
            original = new byte[(contentLength > 2048) ? 2048 : contentLength];
            int i = 0;
            while (i < contentLength) {
                int read = inputStream.read(original, i, original.length - i);
                if (read < 0) {
                    break;
                }
                i += read;
                if (i >= original.length && i < contentLength) {
                    original = Arrays.copyOf(original, i * 2);
                }
                else {
                    continue;
                }
            }
            original = Arrays.copyOf(original, i);
        }
        finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                }
                catch (IOException ex) {
                    throw ex;
                }
            }
            if (outputStream != null) {
                try {
                    outputStream.close();
                }
                catch (IOException ex2) {
                    throw ex2;
                }
            }
        }
        return original;
    }
    
    public static URI getResponderURI(X509Certificate cert) {
        try {
            return getResponderURI(X509CertImpl.toImpl(cert));
        }
        catch (CertificateException ex) {
            return null;
        }
    }
    
    static URI getResponderURI(X509CertImpl x509CertImpl) {
        AuthorityInfoAccessExtension authorityInfoAccessExtension = x509CertImpl.getAuthorityInfoAccessExtension();
        if (authorityInfoAccessExtension == null) {
            return null;
        }
        for (AccessDescription accessDescription : authorityInfoAccessExtension.getAccessDescriptions()) {
            if (accessDescription.getAccessMethod().equals(AccessDescription.Ad_OCSP_Id)) {
                GeneralName accessLocation = accessDescription.getAccessLocation();
                if (accessLocation.getType() == 6) {
                    return ((URIName)accessLocation.getName()).getURI();
                }
                continue;
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    static {
        debug = Debug.getInstance("certpath");
        CONNECT_TIMEOUT = initializeTimeout();
    }
    
    public interface RevocationStatus
    {
        public abstract OCSP.RevocationStatus.CertStatus getCertStatus();
        
        public abstract Date getRevocationTime();
        
        public abstract CRLReason getRevocationReason();
        
        public abstract Map<String, Extension> getSingleExtensions();
        
        public enum CertStatus
        {
            GOOD, 
            REVOKED, 
            UNKNOWN;
        }
    }
}
